/****************************************************************************
 * NCSA HDF                                                                 *
 * Software Development Group                                               *
 * National Center for Supercomputing Applications                          *
 * University of Illinois at Urbana-Champaign                               *
 * 605 E. Springfield, Champaign IL 61820                                   *
 *                                                                          *
 * For conditions of distribution and use, see the accompanying             *
 * hdf/COPYING file.                                                        *
 *                                                                          *
 ****************************************************************************/

/* dfrig.h,v 1.3.2.1 1993/10/26 19:23:17 georgev Exp */

/*-----------------------------------------------------------------------------
 * File:    dfrig.h
 * Purpose: header file for the Raster Image set
 * Invokes: df.h
 * Contents:
 *  Structure definitions: DFRdr, DFRrig
 * Remarks: This is included with user programs which use RIG
 *---------------------------------------------------------------------------*/


#ifndef DFRIG                  /* avoid re-inclusion */
#define DFRIG

/* description record: used to describe image data, palette data etc. */
typedef struct {
    int32 xdim, ydim;          /* dimensions of data */
    DFdi nt;                   /* number type of data */
    int16 ncomponents, interlace; /* data ordering: chunky / planar etc */
    DFdi compr;                        /* compression */
    /* ### Note: compression is currently uniquely described with a tag.
       No data is attached to this tag/ref.  But this capability is
       provided for future expansion, when this tag/ref might point to
       some data needed for decompression, such as the actual encodings */
} DFRdr;

/* structure to hold RIG info */
typedef struct {
    DFdi image;                        /* image */
    DFRdr descimage;           /* image data description */
    DFdi lut;                  /* color look-up table (palette) */
    DFRdr desclut;             /* look-up table description */
    DFdi mattechannel;
    DFRdr descmattechannel;
    int32 xpos, ypos;          /* X-Y position of image on screen */
    float32 aspectratio;         /* ratio of pixel height to width */
    float32 ccngamma, ccnred[3], ccngrren[3], ccnblue[3], ccnwhite[3];
                               /* color correction parameters */
    char *cf;                  /* color format */
} DFRrig;

#endif /*DFRIG*/
